-- This file is a part of Falling Block Game 2.
-- 
-- Copyright (C) 2011 Jared Krinke.
-- 
-- Falling Block Game 2 is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require("Utility/RingBuffer.lua")
require("Utility/LogManager.lua")

LogDisplay = { }

function LogDisplay.new(lines, up)
    local logDisplay = Entity.new()

    logDisplay.lines = { }
    logDisplay.ringBuffer = RingBuffer.new(lines)

    function logDisplay.messageLogged(self, level, message)
        self.ringBuffer:add(message)
    
        local i = 1
    
        -- Update lines
        self.ringBuffer:forEach(function(text)
            local line = self.ringBuffer.count - i + 1
    
            self.lines[line].text = text
            i = i + 1
        end)
    end

    -- Create one text element for each line
    local i = 1

    while i <= lines do
        local line = Element.Text.new()

        if isBoolean(up) and up then
            line.y = i - 1
        else
            line.y = -(i - 1)
        end
    
        logDisplay.elements:add(line)
        logDisplay.lines[i] = line
    
        i = i + 1
    end

    -- Register log handler
    LogManager.addHandler(logDisplay)

    -- Component implementation
    function logDisplay.setLayer(self, layer)
        layer:addChild(self)
    end

    function logDisplay.getPosition(self)
        return self.x, self.y + self.height * lines
    end

    function logDisplay.setPosition(self, x, y)
        self.x = x
        self.y = y - self.height * lines
    end

    function logDisplay.setTextSize(self, width, height)
        self.width = width
        self.height = height
    end

    function logDisplay.getActive(self)
        return false
    end

    function logDisplay.getMinimumSize(self)
        -- TODO: Should this do something different? E.g. actually look at the longest line
        return self.width * 80, self.height * lines
    end

    logDisplay.getDesiredSize = logDisplay.getMinimumSize
    logDisplay.getSize = logDisplay.getMinimumSize

    function logDisplay.setSize(self, width, height)
        -- Ignore non-text size changes for now
    end

    return logDisplay
end

